==> cirq_unstable-0.8.0.dev20200129194008-py3-none-any.whl: cirq/google/op_deserializer_test.py <==
# Copyright 2019 The Cirq Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import List

import pytest
import sympy

import cirq
import cirq.google as cg


@cirq.value_equality
class GateWithAttribute(cirq.SingleQubitGate):

    def __init__(self, val, not_req=None):
        self.val = val
        self.not_req = not_req

    def _value_equality_values_(self):
        return (self.val,)


TEST_CASES = [
    (float, 1.0, {
        'arg_value': {
            'float_value': 1.0
        }
    }),
    (str, 'abc', {
        'arg_value': {
            'string_value': 'abc'
        }
    }),
    (float, 1, {
        'arg_value': {
            'float_value': 1.0
        }
    }),
    (List[bool], [True, False], {
        'arg_value': {
            'bool_values': {
                'values': [True, False]
            }
        }
    }),
    (sympy.Symbol, sympy.Symbol('x'), {
        'symbol': 'x'
    }),
    (float, sympy.Symbol('x') - sympy.Symbol('y'), {
        'func': {
            'type':
            'add',
            'args': [{
                'symbol': 'x'
            }, {
                'func': {
                    'type': 'mul',
                    'args': [{
                        'arg_value': {
                            'float_value': -1.0
                        }
                    }, {
                        'symbol': 'y'
                    }]
                }
            }]
        }
    }),
]


@pytest.mark.parametrize(('val_type', 'val', 'arg_value'), TEST_CASES)
def test_from_proto(val_type, val, arg_value):
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             )
                                         ])
    serialized = {
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': arg_value
        },
        'qubits': [{
            'id': '1_2'
        }]
    }
    q = cirq.GridQubit(1, 2)
    result = deserializer.from_proto_dict(serialized,
                                          arg_function_language='linear')
    assert result == GateWithAttribute(val)(q)


def test_from_proto_required_missing():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             )
                                         ])
    serialized = {
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'not_my_val': {
                'arg_value': {
                    'float_value': 0.125
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    }
    with pytest.raises(Exception, match='my_val'):
        deserializer.from_proto_dict(serialized)


def test_from_proto_unknown_function():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             )
                                         ])
    serialized = {
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {
                'func': {
                    'type': 'UNKNOWN_OPERATION',
                    'args': [
                        {
                            'symbol': 'x'
                        },
                        {
                            'arg_value': {
                                'float_value': -1.0
                            }
                        },
                    ]
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    }
    with pytest.raises(ValueError, match='Unrecognized function type'):
        _ = deserializer.from_proto_dict(serialized)


def test_from_proto_value_type_not_recognized():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             )
                                         ])
    serialized = {
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {
                'arg_value': {},
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    }
    with pytest.raises(ValueError, match='Unrecognized value type'):
        _ = deserializer.from_proto_dict(serialized)


def test_from_proto_function_argument_not_set():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             )
                                         ])
    serialized = {
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {
                'func': {
                    'type': 'mul',
                    'args': [
                        {
                            'symbol': 'x'
                        },
                        {},
                    ]
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    }
    with pytest.raises(ValueError,
                       match='A multiplication argument is missing'):
        _ = deserializer.from_proto_dict(serialized,
                                         arg_function_language='linear')


def test_from_proto_unknown_arg_type():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             )
                                         ])
    serialized = {
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {
                'arg_value': {
                    'what_value': 0.125
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    }
    with pytest.raises(Exception, match='what_value'):
        deserializer.from_proto_dict(serialized)


def test_from_proto_value_func():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                                 value_func=lambda x: x + 1)
                                         ])
    serialized = {
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {
                'arg_value': {
                    'float_value': 0.125
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    }
    q = cirq.GridQubit(1, 2)
    result = deserializer.from_proto_dict(serialized)
    assert result == GateWithAttribute(1.125)(q)


def test_from_proto_not_required_ok():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             ),
                                             cg.DeserializingArg(
                                                 serialized_name='not_req',
                                                 constructor_arg_name='not_req',
                                                 required=False)
                                         ])
    serialized = {
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {
                'arg_value': {
                    'float_value': 0.125
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    }
    q = cirq.GridQubit(1, 2)
    result = deserializer.from_proto_dict(serialized)
    assert result == GateWithAttribute(0.125)(q)


def test_from_proto_missing_required_arg():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             ),
                                             cg.DeserializingArg(
                                                 serialized_name='not_req',
                                                 constructor_arg_name='not_req',
                                                 required=False)
                                         ])
    serialized = {
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'not_req': {
                'arg_value': {
                    'float_value': 0.125
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    }
    with pytest.raises(ValueError):
        deserializer.from_proto_dict(serialized)


def test_from_proto_required_arg_not_assigned():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             ),
                                             cg.DeserializingArg(
                                                 serialized_name='not_req',
                                                 constructor_arg_name='not_req',
                                                 required=False)
                                         ])
    serialized = {
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {}
        },
        'qubits': [{
            'id': '1_2'
        }]
    }
    with pytest.raises(ValueError):
        deserializer.from_proto_dict(serialized)

==> cirq_unstable-0.9.0.dev20200819202824-py3-none-any.whl: cirq/google/op_deserializer_test.py <==
# Copyright 2019 The Cirq Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, List
import pytest
import sympy

from google.protobuf import json_format

import cirq
import cirq.google as cg
from cirq.google.api import v2


def op_proto(json_dict: Dict) -> v2.program_pb2.Operation:
    op = v2.program_pb2.Operation()
    json_format.ParseDict(json_dict, op)
    return op


@cirq.value_equality
class GateWithAttribute(cirq.SingleQubitGate):

    def __init__(self, val, not_req=None):
        self.val = val
        self.not_req = not_req

    def _value_equality_values_(self):
        return (self.val,)


TEST_CASES = [
    (float, 1.0, {
        'arg_value': {
            'float_value': 1.0
        }
    }),
    (str, 'abc', {
        'arg_value': {
            'string_value': 'abc'
        }
    }),
    (float, 1, {
        'arg_value': {
            'float_value': 1.0
        }
    }),
    (List[bool], [True, False], {
        'arg_value': {
            'bool_values': {
                'values': [True, False]
            }
        }
    }),
    (sympy.Symbol, sympy.Symbol('x'), {
        'symbol': 'x'
    }),
    (float, sympy.Symbol('x') - sympy.Symbol('y'), {
        'func': {
            'type':
            'add',
            'args': [{
                'symbol': 'x'
            }, {
                'func': {
                    'type': 'mul',
                    'args': [{
                        'arg_value': {
                            'float_value': -1.0
                        }
                    }, {
                        'symbol': 'y'
                    }]
                }
            }]
        }
    }),
]


@pytest.mark.parametrize(('val_type', 'val', 'arg_value'), TEST_CASES)
def test_from_proto(val_type, val, arg_value):
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             )
                                         ])
    serialized = op_proto({
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': arg_value
        },
        'qubits': [{
            'id': '1_2'
        }]
    })
    q = cirq.GridQubit(1, 2)
    result = deserializer.from_proto(serialized, arg_function_language='linear')
    assert result == GateWithAttribute(val)(q)


def test_from_proto_required_missing():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             )
                                         ])
    serialized = op_proto({
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'not_my_val': {
                'arg_value': {
                    'float_value': 0.125
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    })
    with pytest.raises(Exception, match='my_val'):
        deserializer.from_proto(serialized)


def test_from_proto_unknown_function():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             )
                                         ])
    serialized = op_proto({
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {
                'func': {
                    'type': 'UNKNOWN_OPERATION',
                    'args': [
                        {
                            'symbol': 'x'
                        },
                        {
                            'arg_value': {
                                'float_value': -1.0
                            }
                        },
                    ]
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    })
    with pytest.raises(ValueError, match='Unrecognized function type'):
        _ = deserializer.from_proto(serialized)


def test_from_proto_value_type_not_recognized():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             )
                                         ])
    serialized = op_proto({
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {
                'arg_value': {},
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    })
    with pytest.raises(ValueError, match='Unrecognized value type'):
        _ = deserializer.from_proto(serialized)


def test_from_proto_function_argument_not_set():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             )
                                         ])
    serialized = op_proto({
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {
                'func': {
                    'type': 'mul',
                    'args': [
                        {
                            'symbol': 'x'
                        },
                        {},
                    ]
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    })
    with pytest.raises(ValueError,
                       match='A multiplication argument is missing'):
        _ = deserializer.from_proto(serialized, arg_function_language='linear')


def test_from_proto_value_func():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                                 value_func=lambda x: x + 1)
                                         ])
    serialized = op_proto({
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {
                'arg_value': {
                    'float_value': 0.125
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    })
    q = cirq.GridQubit(1, 2)
    result = deserializer.from_proto(serialized)
    assert result == GateWithAttribute(1.125)(q)


def test_from_proto_not_required_ok():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             ),
                                             cg.DeserializingArg(
                                                 serialized_name='not_req',
                                                 constructor_arg_name='not_req',
                                                 required=False)
                                         ])
    serialized = op_proto({
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {
                'arg_value': {
                    'float_value': 0.125
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    })
    q = cirq.GridQubit(1, 2)
    result = deserializer.from_proto(serialized)
    assert result == GateWithAttribute(0.125)(q)


def test_from_proto_missing_required_arg():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             ),
                                             cg.DeserializingArg(
                                                 serialized_name='not_req',
                                                 constructor_arg_name='not_req',
                                                 required=False)
                                         ])
    serialized = op_proto({
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'not_req': {
                'arg_value': {
                    'float_value': 0.125
                }
            }
        },
        'qubits': [{
            'id': '1_2'
        }]
    })
    with pytest.raises(ValueError):
        deserializer.from_proto(serialized)


def test_from_proto_required_arg_not_assigned():
    deserializer = cg.GateOpDeserializer(serialized_gate_id='my_gate',
                                         gate_constructor=GateWithAttribute,
                                         args=[
                                             cg.DeserializingArg(
                                                 serialized_name='my_val',
                                                 constructor_arg_name='val',
                                             ),
                                             cg.DeserializingArg(
                                                 serialized_name='not_req',
                                                 constructor_arg_name='not_req',
                                                 required=False)
                                         ])
    serialized = op_proto({
        'gate': {
            'id': 'my_gate'
        },
        'args': {
            'my_val': {}
        },
        'qubits': [{
            'id': '1_2'
        }]
    })
    with pytest.raises(ValueError):
        deserializer.from_proto(serialized)
